/*---[ firestarter.c ]------------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The main application file
 *--------------------------------------------------------------------*/

#include <config.h>
#include <gnome.h>
#include <applet-widget.h>
#include <errno.h>

#include "druid.h"
#include "service.h"
#include "preferences.h"

#include "grey1.xpm"
#include "blue1.xpm"
#include "red1.xpm"
#include "rundruid.xpm"

GtkWidget *clist;
GtkWidget *vbox;
GtkWidget *panelpic;

gboolean running = TRUE;
gboolean logenabled = TRUE;

gint parse (char *);
void set_panel_pic (gchar *pixmap[]);


/*---[ hide_window_cb ]-----------------------------------------------
 * Hides the main window, either when "destroyed" or applet clicked.
 *--------------------------------------------------------------------*/
gint
hide_window_cb (GtkWidget *window, gpointer data)
{
	if (GTK_WIDGET_VISIBLE (window))
		gtk_widget_hide (window);
	else
		gtk_widget_show (window);

	set_panel_pic (blue1_xpm);
	return TRUE;
}

/*---[ savelog ]------------------------------------------------------
 * Saves the log to a disk file, called from savelog_filsel_cb
 *--------------------------------------------------------------------*/
void
savelog (GtkWidget *widget, gpointer data) 
{
	gchar *text;
	gint i = 0;
	FILE *f;

	gchar *savefile;

	savefile = gtk_file_selection_get_filename (GTK_FILE_SELECTION(data));

	f = fopen (savefile, "w");

	while (gtk_clist_get_text (GTK_CLIST (clist), i, 0, &text)) {
		fprintf (f, "%s\t\t", text);
		
		gtk_clist_get_text (GTK_CLIST (clist), i, 1, &text);
		fprintf (f, "%s\t\t", text);

		gtk_clist_get_text (GTK_CLIST (clist), i, 2, &text);
		fprintf (f, "%s\t\t", text);

		gtk_clist_get_text (GTK_CLIST (clist), i, 3, &text);
		fprintf (f, "%s\n", text);

		i++;
	}
	fclose (f);
}

/*---[ savelog_filesel_cb ]-------------------------------------------
 * Creates a file selection dialog, passed to savelog
 *--------------------------------------------------------------------*/
static void
savelog_filesel_cb (GtkWidget *widget, gpointer data)
{
	if (logenabled) {
		GtkWidget *filesel;

		filesel = gtk_file_selection_new N_("Select file to save log to");

		gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(filesel)->ok_button), "clicked",
				    GTK_SIGNAL_FUNC (savelog), (gpointer) filesel);

		gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(filesel)->ok_button), "clicked",
					   GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) filesel);

		gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(filesel)->cancel_button), "clicked",
					   GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) filesel);

		gtk_widget_show (filesel);
	}
}

/*---[ stop_firewall ]------------------------------------------------
 * Flushes, zeroes and sets the default policy of the chains
 *--------------------------------------------------------------------*/
static void 
stop_firewall_cb (GtkWidget *widget, gpointer data)
{
	if (geteuid ()) {
		GtkWidget *dialog=gnome_error_dialog N_("You must be root to manipulate the firewall.");
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	}
	else {
		system ("/sbin/ipchains -X; /sbin/ipchains -F; /sbin/ipchains -Z;");
		system ("/sbin/ipchains -P input ACCEPT; /sbin/ipchains -P forward ACCEPT; /sbin/ipchains -P output ACCEPT;");
		g_print N_("Firewall stopped, network traffic is now flowing freely\n");

		running = FALSE;
		set_panel_pic (grey1_xpm);
	}
}

/*---[ restart_firewall_cb ]------------------------------------------
 * Executes the firewall script
 *--------------------------------------------------------------------*/
static void 
restart_firewall_cb (GtkWidget *widget, gpointer data)
{
	if (geteuid ()) {
		GtkWidget *dialog=gnome_error_dialog N_("You must be root to manipulate the firewall.");
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	}
	else {
		gchar *commandline = g_strjoin (NULL, "/bin/sh ",
						gnome_config_get_string ("/firestarter/Files/scriptfile=/usr/local/firestarter/firewall.sh"),
						NULL);
		system (commandline);
		g_print N_("Firewall script restarted\n");
		running = TRUE;

		set_panel_pic (blue1_xpm);
	}
}

/*---[ clear_button_cb ]----------------------------------------------
 * Clears the log CList
 *--------------------------------------------------------------------*/
static void
clear_button_cb (gpointer data)
{
	if (running)
		set_panel_pic (blue1_xpm);
	gtk_clist_clear (GTK_CLIST (clist));
}

/*---[ show_entire_log_cb ]-------------------------------------------
 * Reads the entire log file into the CList
 *--------------------------------------------------------------------*/
void
show_entire_log_cb (gpointer data)
{
	if (logenabled) {
		char buf[512];
		char *logpath = gnome_config_get_string ("/firestarter/Files/logfile=/var/log/messages");
		FILE *f;

		f = fopen (logpath, "r");

		gtk_clist_clear (GTK_CLIST (clist));
		while (fgets (buf, 512, f) != NULL) {
			if (strstr (buf, "DENY"))
				parse (buf);
		}
		fclose (f);
	}
}

/*---[ timeout_cb ]---------------------------------------------------
 * Polls the logfile every 700ms for change, if change, parse lines
 *--------------------------------------------------------------------*/
int
timeout_cb (gpointer file)
{
	char buf[512];

	if (fgets (buf, 512, file) != NULL) {
		if (strstr (buf, "DENY"))
			if (parse (buf))
				set_panel_pic (red1_xpm);
		}
	return TRUE;
}

/*---[ parse ]--------------------------------------------------------
 * Parses *line. Extracts port, remoteip, service, time and protocol
 *--------------------------------------------------------------------*/
int
parse (char *line)
{
	char *key = "DENY";
	gchar *loc1;
	gchar *loc2;
	gchar *data[4] = {"port", "remoteip", "service", "time"};
	gchar *proto;

	data[3] = g_strndup (line, 12);
	loc1 = (char *)strstr (line, key);

	if (loc1 != NULL) {
		/* Determine the protocol used, not in use at the moment */
		if (strstr (loc1, "PROTO=") != NULL)
			loc1 = (char *)strstr (loc1, "PROTO=");
		else
			return FALSE;
		loc1 += 6*(sizeof (char));
		proto = g_strndup (loc1, 2);
		/* do an atoi to remove possible trailing garbage */
		proto = g_strdup_printf ("%d", atoi (proto));

		/* Determine the senders IP */
		loc1 += 2*(sizeof (char));
		if (strchr (loc1, ':') != NULL)
			loc2 = (char *)strchr (loc1, ':');
		else
			return FALSE;
		data[1] = g_strndup (loc1, loc2-loc1);
		g_strstrip (data[1]);

		/* Determine the port the packet was sent to */
		if (strrchr (loc1, ':') != NULL)
			loc1 = (char *)strrchr (loc1, ':');
		else
			return FALSE;
		loc1 += (sizeof (char));
		data[0] = g_strndup (loc1, 5);
		data[0] = g_strdup_printf ("%d", atoi (data[0]));

		/* Determine the service used, based on used port */
		data[2] = service (atoi (data[0]));
		gtk_clist_append (GTK_CLIST (clist), data);
		return TRUE;
	}
	return FALSE;
}

/*---[ about_cb ]-----------------------------------------------------
 * Creates the about dialog
 *--------------------------------------------------------------------*/
static void 
about_cb (GtkWidget *widget, gpointer data)
{
	static GtkWidget *dialog = NULL;

	if (dialog != NULL) {
		g_assert (GTK_WIDGET_REALIZED (dialog));
		gdk_window_show (dialog->window);
		gdk_window_raise (dialog->window);
	}
	else {
		const gchar *authors[] = {"Tomas Junnonen <majix@sci.fi>",
					  NULL};

		dialog = gnome_about_new ("FireStarter", VERSION,
					  "(C) 2000 Tomas Junnonen", authors,
					 _("An all-in-one Linux firewall utility for GNOME"), NULL);

		gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
				    GTK_SIGNAL_FUNC (gtk_widget_destroyed), &dialog);

		gtk_widget_show (dialog);
	}
}

/*---[ set_panel_pic ]-----------------------------------------------------
 * Sets the picture on the panel to *pixmap
 *--------------------------------------------------------------------*/
void
set_panel_pic (gchar *pixmap[])
{
	if (gnome_config_get_bool ("/firestarter/Applet/dock=TRUE")) {
		gtk_container_remove (GTK_CONTAINER (vbox), panelpic);
		panelpic = gnome_pixmap_new_from_xpm_d (pixmap);
		gtk_box_pack_end (GTK_BOX (vbox), panelpic, FALSE, FALSE, 2);
		gtk_widget_show (panelpic);
	}
}

/* Create the menus and toolbar */
static GnomeUIInfo firewall_menu [] = {
	GNOMEUIINFO_ITEM (N_("_Run firewall wizard"), NULL, rundruid, rundruid_xpm),
//	GNOMEUIINFO_ITEM_STOCK (N_("_Run firewall wizard"), NULL, rundruid, GNOME_STOCK_PIXMAP_EXEC),
	GNOMEUIINFO_ITEM_NONE (N_("R_estart firewall"), NULL, restart_firewall_cb),
	GNOMEUIINFO_ITEM_NONE (N_("_Stop firewall"), NULL, stop_firewall_cb),
	GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_cb, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM (gtk_main_quit, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo log_menu [] = {  
	GNOMEUIINFO_ITEM_STOCK (N_("_Clear log"), NULL, clear_button_cb, GNOME_STOCK_PIXMAP_CLEAR),
	GNOMEUIINFO_ITEM_STOCK (N_("S_how entire log"), NULL, show_entire_log_cb, GNOME_STOCK_PIXMAP_UNDO),
	GNOMEUIINFO_ITEM_STOCK (N_("_Save log to file"), NULL, savelog_filesel_cb, GNOME_STOCK_PIXMAP_SAVE),
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu [] = {  
	GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menu [] = {
	GNOMEUIINFO_SUBTREE (N_("_Firewall"), firewall_menu),
	GNOMEUIINFO_SUBTREE (N_("_Log"), log_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo toolbar [] = {
	GNOMEUIINFO_ITEM (NULL, N_("Run firewall wizard"), rundruid, rundruid_xpm),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK (NULL, N_("Clear log display"), clear_button_cb, GNOME_STOCK_PIXMAP_CLEAR),
  	GNOMEUIINFO_ITEM_STOCK (NULL, N_("Show entire firewall log"), show_entire_log_cb, GNOME_STOCK_PIXMAP_UNDO),
  	GNOMEUIINFO_ITEM_STOCK (NULL, N_("Save log to file"), savelog_filesel_cb, GNOME_STOCK_PIXMAP_SAVE),
	GNOMEUIINFO_SEPARATOR,
  	GNOMEUIINFO_ITEM_STOCK (NULL, N_("Restart firewall"), restart_firewall_cb, GNOME_STOCK_PIXMAP_REFRESH),
  	GNOMEUIINFO_ITEM_STOCK (NULL, N_("Stop firewall"), stop_firewall_cb, GNOME_STOCK_PIXMAP_STOP),
	GNOMEUIINFO_END
};


/*---[ main ]---------------------------------------------------------
 * The main function, this is where it all begins
 *--------------------------------------------------------------------*/
int
main (int argc, char* argv[])
{
	GtkWidget *applet;
	GtkWidget *eventbox;
	GtkWidget *frame;

	GtkWidget *window;
	GtkWidget *scrolled_win;

	/* *titles is used to determinte the max width, in pixels, the clist columns are at run time */
	gchar *titles[4] = {"00000", "000.000.000.000", "00000000", "000 00 00.000"};
	gint width, totwidth = 0;
	gint i;

	char buf[512];
	char *logpath;
	FILE *file;             

	applet_widget_init(PACKAGE, VERSION, argc, argv, NULL, 0, NULL);
	
	if (gnome_config_get_bool ("/firestarter/Applet/dock=TRUE")) {
		applet = applet_widget_new (PACKAGE);
	}

	window = gnome_app_new (PACKAGE, "FireStarter");

	logpath = gnome_config_get_string ("/firestarter/Files/logfile=/var/log/messages");

/* Set up the applet, unless registry value dock=false */
	if (gnome_config_get_bool ("/firestarter/Applet/dock=TRUE")) {
		eventbox = gtk_event_box_new ();
		gtk_widget_set_events (eventbox, GDK_BUTTON_PRESS_MASK);
		gtk_signal_connect_object (GTK_OBJECT (eventbox), "button_press_event",
				    	   GTK_SIGNAL_FUNC (hide_window_cb), GTK_OBJECT (window));
		applet_widget_add (APPLET_WIDGET (applet), eventbox);

		frame = gtk_frame_new (NULL);
		gtk_container_add (GTK_CONTAINER (eventbox), frame);

		vbox = gtk_vbox_new (FALSE, 5);
		gtk_container_add (GTK_CONTAINER (frame), vbox);

		panelpic = gnome_pixmap_new_from_xpm_d (blue1_xpm);
		gtk_box_pack_start (GTK_BOX (vbox), panelpic, FALSE, FALSE, 2);

		applet_widget_register_stock_callback (APPLET_WIDGET (applet), "restart",
						       GNOME_STOCK_MENU_REFRESH, _("Restart firewall"),
						       GTK_SIGNAL_FUNC (restart_firewall_cb), NULL);

		applet_widget_register_stock_callback (APPLET_WIDGET (applet), "stop",
						       GNOME_STOCK_MENU_STOP, _("Stop firewall"),
						       GTK_SIGNAL_FUNC (stop_firewall_cb), NULL);

		applet_widget_register_stock_callback (APPLET_WIDGET (applet), "pref",
						       GNOME_STOCK_MENU_PREF, _("Preferences"),
						       GTK_SIGNAL_FUNC (preferences_cb), NULL);

		applet_widget_register_stock_callback (APPLET_WIDGET (applet), "about",
						       GNOME_STOCK_MENU_ABOUT, _("About"),
						       GTK_SIGNAL_FUNC (about_cb), NULL);
	}
/* Set up the window */
	if (gnome_config_get_bool ("/firestarter/Applet/dock=TRUE")) {
		gtk_signal_connect (GTK_OBJECT (window), "delete_event",
				    GTK_SIGNAL_FUNC (gtk_widget_hide), NULL);
	}
	else {
		gtk_signal_connect (GTK_OBJECT (window), "delete_event",
				    GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
	}

	gnome_app_create_toolbar_with_data (GNOME_APP (window), toolbar, window);
	gnome_app_create_menus_with_data (GNOME_APP (window), menu, window);

	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
					GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
	gnome_app_set_contents (GNOME_APP (window), scrolled_win);

	clist = gtk_clist_new_with_titles (4, titles);
	gtk_container_add (GTK_CONTAINER (scrolled_win), clist);

	/* Check if user has read access to the log file */
	if (fopen (logpath, "r") == NULL) {
		gchar *string;
		GtkWidget *dialog;
	
		string = g_strjoin (NULL, N_("You do not have permission to read the file \""), logpath,
				    N_("\".\nFirewall log monitoring disabled.\n\nYou can configure the"
				    " path to the log file in the \"preferences\" section."),  NULL);
		dialog = gnome_error_dialog (string);
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		logenabled = FALSE;
		gtk_widget_set_sensitive (GTK_WIDGET (clist), FALSE);
	}
	else {
		file = fopen (logpath, "r");
		/* Move to the end of the file */
		while (fgets (buf, 512, file) != NULL) continue;
		gtk_timeout_add (500, timeout_cb, file);
	}

	gtk_widget_show_all (window);
	/* The widths of the clist columns and the window is determinted dynamically from *titles.
	   The widths vary depending on the theme and the font used */
	for (i = 0; i < 4; i++) {
		totwidth +=  width = gtk_clist_optimal_column_width (GTK_CLIST (clist), i);
		gtk_clist_set_column_width (GTK_CLIST (clist), i, width);
	}
	gtk_window_set_default_size (GTK_WINDOW (window), totwidth, 300);

	gtk_clist_set_column_title (GTK_CLIST (clist), 0, N_("Port"));
	gtk_clist_set_column_title (GTK_CLIST (clist), 1, N_("IP of sender"));
	gtk_clist_set_column_title (GTK_CLIST (clist), 2, N_("Service"));
	gtk_clist_set_column_title (GTK_CLIST (clist), 3, N_("Time"));

	mkdir ("/usr/local/firestarter", 00700);

	if (gnome_config_get_bool ("/firestarter/Applet/dock=TRUE")) {
		gtk_widget_show_all (applet);
		applet_widget_gtk_main ();
	}
	else
		gtk_main ();

	return 0;
}
