/*---[ preferences.c ]------------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The preferences notebook window
 *--------------------------------------------------------------------*/

#include "preferences.h"

GtkWidget *scriptentry;
GtkWidget *logentry;
GtkWidget *appletdock;

/*---[ save_preferences.c ]-------------------------------------------
 * Save the preferences to the gnome registry
 *--------------------------------------------------------------------*/
static void
save_preferences (GnomePropertyBox *property_box, gint page_num, gint num)
{
	gnome_config_set_string ("/firestarter/Files/logfile",
				 gtk_entry_get_text (GTK_ENTRY (logentry)));

	gnome_config_set_string ("/firestarter/Files/scriptfile",
				 gtk_entry_get_text (GTK_ENTRY (scriptentry)));

	gnome_config_set_bool ("/firestarter/Applet/dock",
			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (appletdock)));

	gnome_config_sync ();
}

/*---[ preferences_cb ]-----------------------------------------------
 * Create the preferences property_box
 *--------------------------------------------------------------------*/
void 
preferences_cb (GtkWidget *widget, gpointer data)
{
	static GtkWidget *pbox;
	GtkWidget *page;
	GtkWidget *tab;
	GtkWidget *table;
	GtkWidget *label;

	/* Check that the window is not already created. Do not fix indentation */
	if (pbox != NULL) {
		g_assert (GTK_WIDGET_REALIZED (pbox));
		gdk_window_show (pbox->window);
		gdk_window_raise (pbox->window);
	}
	else {

	tab = gtk_label_new _("General");
	page = gtk_vbox_new (FALSE, 0);

	table = gtk_table_new (3, 2, FALSE);
	gtk_box_pack_start (GTK_BOX (page), table, FALSE, FALSE, 0);
	gtk_table_set_row_spacings (GTK_TABLE(table), 0);
	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
                                 
	label = gtk_label_new _("Firewall log file:");
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	logentry = gtk_entry_new ();
	gtk_widget_set_usize (logentry, 250, 20);
	gtk_entry_set_text (GTK_ENTRY (logentry),
			    gnome_config_get_string ("/firestarter/Files/logfile=/var/log/messages"));

	gtk_table_attach (GTK_TABLE (table), logentry, 1, 2, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	label = gtk_label_new _("Firewall script file:");
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	scriptentry = gtk_entry_new ();
	gtk_entry_set_text (GTK_ENTRY (scriptentry),
			    gnome_config_get_string ("/firestarter/Files/scriptfile=/usr/local/firestarter/firewall.sh"));
	gtk_table_attach (GTK_TABLE (table), scriptentry, 1, 2, 1, 2,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	appletdock = gtk_check_button_new_with_label _("Display status indicator on the GNOME panel (requires restart)");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (appletdock), gnome_config_get_bool ("/firestarter/Applet/dock=TRUE"));
	gtk_box_pack_start (GTK_BOX (page), appletdock, FALSE, FALSE, 5);

	pbox = gnome_property_box_new ();
	gnome_property_box_append_page (GNOME_PROPERTY_BOX (pbox), page, tab);

	gtk_signal_connect_object (GTK_OBJECT (logentry), "changed",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox)); 

	gtk_signal_connect_object (GTK_OBJECT (scriptentry), "changed",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox)); 

	gtk_signal_connect_object (GTK_OBJECT (appletdock), "toggled",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed), GTK_OBJECT (pbox)); 

	gtk_signal_connect (GTK_OBJECT (pbox), "apply",
			    GTK_SIGNAL_FUNC (save_preferences), NULL);

	gtk_signal_connect (GTK_OBJECT (pbox), "destroy",
			    GTK_SIGNAL_FUNC (gtk_widget_destroyed), &pbox);

	gtk_widget_show_all (pbox);
	}
}
